@@ -9,8 +9,7 @@ __all__ = [
 # IMPORTS #
 ###########
 
-
-# Major
+# Full
 
 import networkx as nx
 import numpy as np
@@ -18,7 +17,7 @@ import numpy.linalg as npl
 import scipy.integrate as spi
 import scipy.stats as sps
 
-# Minor
+# Partial
 
 from copy import (
     deepcopy
@@ -60,7 +59,6 @@ from .validation import *
 # CLASSES #
 ###########
 
-
 @aliased
 class MarkovChain(metaclass=BaseClass):
 
@@ -88,7 +86,7 @@ class MarkovChain(metaclass=BaseClass):
                 else:
                     states = validate_state_names(states, p.shape[0])
 
-            except Exception as e:
+            except Exception as e:  # pragma: no cover
                 raise generate_validation_error(e, trace()) from None
 
         size = p.shape[0]
@@ -816,7 +814,7 @@ class MarkovChain(metaclass=BaseClass):
             state1 = validate_state(state1, self._states)
             state2 = validate_state(state2, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         a1 = self.accessibility_matrix[state1, state2] != 0
@@ -825,14 +823,14 @@ class MarkovChain(metaclass=BaseClass):
 
         return result
 
-    def closest_reversible(self, distribution: tnumeric, weighted: bool = False) -> omc:
+    def closest_reversible(self, distribution: onumeric = None, weighted: bool = False) -> tmc:
 
         """
         The method computes the closest reversible of the Markov chain.
 
         | **Notes:** the algorithm is described in `Computing the nearest reversible Markov chain (Nielsen & Weber, 2015) <http://doi.org/10.1002/nla.1967>`_.
 
-        :param distribution: the distribution of the states.
+        :param distribution: the distribution of the states (if omitted, the states are assumed to be uniformly distributed).
         :param weighted: a boolean indicating whether to use the weighted Frobenius norm (by default, False).
         :return: a Markov chain representing the closest reversible.
         :raises ValidationError: if any input argument is not compliant.
@@ -841,22 +839,29 @@ class MarkovChain(metaclass=BaseClass):
 
         try:
 
-            distribution = validate_vector(distribution, 'stochastic', False, size=self._size)
+            if distribution is None:
+                distribution = np.ones(self._size, dtype=float) / self._size
+            else:
+                distribution = validate_vector(distribution, 'stochastic', False, size=self._size)
+
             weighted = validate_boolean(weighted)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
-        non_zeros = np.count_nonzero(distribution)
-        zeros = len(distribution) - non_zeros
+        zeros = len(distribution) - np.count_nonzero(distribution)
 
         if weighted and zeros > 0:
             raise ValidationError('If the weighted Frobenius norm is used, the distribution must not contain zero-valued probabilities.')
 
-        p, error_message = closest_reversible(self._p, distribution, weighted)
+        if self.is_reversible:
+            p = self._p
+        else:
 
-        if error_message is not None:
-            raise ValueError(error_message)
+            p, error_message = closest_reversible(self._p, distribution, weighted)
+
+            if error_message is not None:
+                raise ValueError(error_message)
 
         mc = MarkovChain(p, self._states)
 
@@ -883,7 +888,7 @@ class MarkovChain(metaclass=BaseClass):
             states1 = validate_states(states1, self._states, 'subset', True)
             states2 = validate_states(states2, self._states, 'subset', True)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         intersection = np.intersect1d(states1, states2)
@@ -912,7 +917,7 @@ class MarkovChain(metaclass=BaseClass):
 
             state = validate_state(state, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         value = self._p[state, :]
@@ -935,15 +940,16 @@ class MarkovChain(metaclass=BaseClass):
             rewards = validate_rewards(rewards, self._size)
             steps = validate_integer(steps, lower_limit=(0, True))
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         original_rewards = np.copy(rewards)
+        value = np.copy(rewards)
 
-        for i in range(steps):
-            rewards = original_rewards + np.dot(rewards, self._p)
+        for _ in range(steps):
+            value = original_rewards + np.dot(value, self._p)
 
-        return rewards
+        return value
 
     def expected_transitions(self, steps: int, initial_distribution: onumeric = None) -> oarray:
 
@@ -965,7 +971,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 initial_distribution = validate_vector(initial_distribution, 'stochastic', False, size=self._size)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if steps <= self._size:
@@ -973,7 +979,7 @@ class MarkovChain(metaclass=BaseClass):
             pi = initial_distribution
             p_sum = initial_distribution
 
-            for i in range(steps - 1):
+            for _ in range(steps - 1):
                 pi = np.dot(pi, self._p)
                 p_sum += pi
 
@@ -1024,7 +1030,7 @@ class MarkovChain(metaclass=BaseClass):
             if first_passage_states is not None:
                 first_passage_states = validate_states(first_passage_states, self._states, 'regular', True)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         value = first_passage_probabilities(self, steps, initial_state, first_passage_states)
@@ -1055,7 +1061,7 @@ class MarkovChain(metaclass=BaseClass):
             rewards = validate_rewards(rewards, self._size)
             steps = validate_integer(steps, lower_limit=(0, True))
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if self._size == 2:
@@ -1088,7 +1094,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 targets = validate_states(targets, self._states, 'regular', True)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         value = hitting_probabilities(self, targets)
@@ -1112,7 +1118,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 targets = validate_states(targets, self._states, 'regular', True)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         value = hitting_times(self, targets)
@@ -1133,7 +1139,7 @@ class MarkovChain(metaclass=BaseClass):
 
             state = validate_state(state, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         result = state in self._absorbing_states_indices
@@ -1156,7 +1162,7 @@ class MarkovChain(metaclass=BaseClass):
             state_target = validate_state(state_target, self._states)
             state_origin = validate_state(state_origin, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         result = self.accessibility_matrix[state_origin, state_target] != 0
@@ -1177,7 +1183,7 @@ class MarkovChain(metaclass=BaseClass):
 
             state = validate_state(state, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         result = state in self._cyclic_states_indices
@@ -1198,7 +1204,7 @@ class MarkovChain(metaclass=BaseClass):
 
             state = validate_state(state, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         result = state in self._recurrent_states_indices
@@ -1219,7 +1225,7 @@ class MarkovChain(metaclass=BaseClass):
 
             state = validate_state(state, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         result = state in self._transient_states_indices
@@ -1241,7 +1247,7 @@ class MarkovChain(metaclass=BaseClass):
 
             partitions = validate_partitions(partitions, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if self._size == 2:
@@ -1291,7 +1297,7 @@ class MarkovChain(metaclass=BaseClass):
             origins = validate_states(origins, self._states, 'subset', True)
             targets = validate_states(targets, self._states, 'subset', True)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         value = mean_first_passage_times_between(self, origins, targets)
@@ -1316,7 +1322,7 @@ class MarkovChain(metaclass=BaseClass):
             if targets is not None:
                 targets = validate_states(targets, self._states, 'regular', True)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         value = mean_first_passage_times_to(self, targets)
@@ -1377,7 +1383,7 @@ class MarkovChain(metaclass=BaseClass):
             jump = validate_integer(jump, lower_limit=(0, True))
             cutoff_type = validate_enumerator(cutoff_type, ['natural', 'traditional'])
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if cutoff_type == 'traditional':
@@ -1419,7 +1425,7 @@ class MarkovChain(metaclass=BaseClass):
             include_initial = validate_boolean(include_initial)
             output_indices = validate_boolean(output_indices)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         prediction = list()
@@ -1429,7 +1435,7 @@ class MarkovChain(metaclass=BaseClass):
 
         current_state = initial_state
 
-        for i in range(steps):
+        for _ in range(steps):
             d = self._p[current_state, :]
             d_max = np.argwhere(d == np.max(d))
 
@@ -1461,7 +1467,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 hyperparameter = validate_hyperparameter(hyperparameter, self._size)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         lps = np.zeros(self._size, dtype=float)
@@ -1503,7 +1509,7 @@ class MarkovChain(metaclass=BaseClass):
             include_initial = validate_boolean(include_initial)
             output_last = validate_boolean(output_last)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         distributions = np.zeros((steps, self._size), dtype=float)
@@ -1539,7 +1545,7 @@ class MarkovChain(metaclass=BaseClass):
 
             state = validate_state(state, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         value = sensitivity(self, state)
@@ -1567,7 +1573,7 @@ class MarkovChain(metaclass=BaseClass):
 
             time_points = validate_time_points(time_points)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if not self.is_ergodic:
@@ -1681,7 +1687,7 @@ class MarkovChain(metaclass=BaseClass):
 
             time_points = validate_time_points(time_points)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if not self.is_ergodic:
@@ -1776,7 +1782,7 @@ class MarkovChain(metaclass=BaseClass):
 
             boundary_condition = validate_boundary_condition(boundary_condition)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         p, _ = bounded(self._p, boundary_condition)
@@ -1810,7 +1816,7 @@ class MarkovChain(metaclass=BaseClass):
     def to_dictionary(self) -> tmc_dict:
 
         """
-        The method returns a dictionary representing the Markov chain.
+        The method returns a dictionary representing the Markov chain transitions.
 
         :return: a dictionary.
         """
@@ -1823,15 +1829,12 @@ class MarkovChain(metaclass=BaseClass):
 
         return d
 
-    @alias('to_graph')
-    def to_directed_graph(self, multi: bool = True) -> tgraphs:
+    def to_graph(self, multi: bool = False) -> tgraphs:
 
         """
         The method returns a directed graph representing the Markov chain.
 
-        | **Aliases:** to_graph
-
-        :param multi: a boolean indicating whether the graph is allowed to define multiple edges between two nodes (by default, True).
+        :param multi: a boolean indicating whether the graph is allowed to define multiple edges between two nodes (by default, False).
         :return: a directed graph.
         :raises ValidationError: if any input argument is not compliant.
         """
@@ -1840,7 +1843,7 @@ class MarkovChain(metaclass=BaseClass):
 
             multi = validate_boolean(multi)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if multi:
@@ -1867,19 +1870,19 @@ class MarkovChain(metaclass=BaseClass):
 
             file_path = validate_string(file_path)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         file_extension = get_file_extension(file_path)
 
-        if file_extension not in ['csv', 'json', 'txt']:
+        if file_extension not in ['.csv', '.json', '.txt']:
             raise ValidationError('Only csv, json and plain text files are supported.')
 
         d = self.to_dictionary()
 
-        if file_extension == 'csv':
+        if file_extension == '.csv':
             write_csv(d, file_path)
-        elif file_extension == 'json':
+        elif file_extension == '.json':
             write_json(d, file_path)
         else:
             write_txt(d, file_path)
@@ -1901,7 +1904,7 @@ class MarkovChain(metaclass=BaseClass):
 
             inertial_weights = validate_vector(inertial_weights, 'unconstrained', True, size=self._size)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         p, _ = lazy(self._p, inertial_weights)
@@ -1909,6 +1912,18 @@ class MarkovChain(metaclass=BaseClass):
 
         return mc
 
+    def to_matrix(self) -> tarray:
+
+        """
+        The method returns a transition matrix representing the Markov chain.
+
+        :return: a transition matrix.
+        """
+
+        m = np.copy(self._p)
+
+        return m
+
     def to_subchain(self, states: tstates) -> tmc:
 
         """
@@ -1924,7 +1939,7 @@ class MarkovChain(metaclass=BaseClass):
 
             states = validate_states(states, self._states, 'subset', True)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         p, states, error_message = sub(self._p, self._states, self.adjacency_matrix, states)
@@ -1952,7 +1967,7 @@ class MarkovChain(metaclass=BaseClass):
             state_target = validate_state(state_target, self._states)
             state_origin = validate_state(state_origin, self._states)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         tp = self._p[state_origin, state_target]
@@ -1991,7 +2006,7 @@ class MarkovChain(metaclass=BaseClass):
 
             output_indices = validate_boolean(output_indices)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         walk = list()
@@ -2001,7 +2016,7 @@ class MarkovChain(metaclass=BaseClass):
 
         current_state = initial_state
 
-        for i in range(steps):
+        for _ in range(steps):
 
             w = self._p[current_state, :]
             current_state = rng.choice(self._size, size=1, p=w).item()
@@ -2029,7 +2044,7 @@ class MarkovChain(metaclass=BaseClass):
 
             walk = validate_states(walk, self._states, 'walk', False)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         p = 0.0
@@ -2120,7 +2135,7 @@ class MarkovChain(metaclass=BaseClass):
                 else:
                     k = validate_float(k, lower_limit=(0.0, True))
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if approximation_type == 'adda-cooper':
@@ -2263,7 +2278,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 states = validate_state_names(states, {p.shape[0], q.shape[0]}.pop())
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         if p.shape[0] != q.shape[0]:
@@ -2309,7 +2324,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 quadrature_interval = validate_interval(quadrature_interval)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         size = len(possible_states)
@@ -2518,7 +2533,7 @@ class MarkovChain(metaclass=BaseClass):
 
             confidence_level = validate_float(confidence_level, lower_limit=(0.0, False), upper_limit=(1.0, False))
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         size = len(possible_states)
@@ -2616,7 +2631,7 @@ class MarkovChain(metaclass=BaseClass):
 
             d = validate_dictionary(d)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         states = [key[0] for key in d.keys() if key[0] == key[1]]
@@ -2637,6 +2652,40 @@ class MarkovChain(metaclass=BaseClass):
 
         return mc
 
+    @staticmethod
+    def from_graph(graph: tgraphs) -> tmc:
+
+        """
+        The method generates a Markov chain from the given directed graph.
+
+        :return: a Markov chain.
+        :raises ValueError: if the transition matrix defined by the directed graph is not valid.
+        :raises ValidationError: if any input argument is not compliant.
+        """
+
+        try:
+
+            graph = validate_graph(graph)
+
+        except Exception as e:  # pragma: no cover
+            raise generate_validation_error(e, trace()) from None
+
+        states = list(graph.nodes)
+        size = len(states)
+
+        p = np.zeros((size, size), dtype=float)
+
+        for state_from, weights in graph.adjacency():
+            i = states.index(state_from)
+            for state_to, data in weights.items():
+                j = states.index(state_to)
+                w = data['weight']
+                p[i, j] = w
+
+        mc = MarkovChain(p, states)
+
+        return mc
+
     @staticmethod
     def from_file(file_path: str) -> tmc:
 
@@ -2671,22 +2720,22 @@ class MarkovChain(metaclass=BaseClass):
 
             file_path = validate_string(file_path)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         file_extension = get_file_extension(file_path)
 
-        if file_extension not in ['csv', 'json', 'txt']:
+        if file_extension not in ['.csv', '.json', '.txt']:
             raise ValidationError('Only csv, json and plain text files are supported.')
 
-        if file_extension == 'csv':
+        if file_extension == '.csv':
             d = read_csv(file_path)
-        elif file_extension == 'json':
+        elif file_extension == '.json':
             d = read_json(file_path)
         else:
             d = read_txt(file_path)
 
-        states = sorted(list(set(sum(d.keys(), ()))))
+        states = [key[0] for key in d.keys() if key[0] == key[1]]
         size = len(states)
 
         if size < 2:
@@ -2725,11 +2774,12 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 states = validate_state_names(states, m.shape[0])
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         m = np.interp(m, (np.min(m), np.max(m)), (0.0, 1.0))
         m /= np.sum(m, axis=1, keepdims=True)
+        m = np.nan_to_num(m, 1.0 / m.shape[0])
 
         return MarkovChain(m, states)
 
@@ -2756,7 +2806,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 states = validate_state_names(states, size)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         p, _ = gamblers_ruin(size, w)
@@ -2785,7 +2835,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 states = validate_state_names(states, size)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         p = np.eye(size, dtype=float)
@@ -2825,7 +2875,7 @@ class MarkovChain(metaclass=BaseClass):
             else:
                 mask = validate_mask(mask, size)
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         p, error_message = random(rng, size, zeros, mask)
@@ -2856,7 +2906,7 @@ class MarkovChain(metaclass=BaseClass):
             n = validate_integer(n, lower_limit=(1, False))
             model = validate_enumerator(model, ['bernoulli-laplace', 'ehrenfest'])
 
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             raise generate_validation_error(e, trace()) from None
 
         p, states, _ = urn_model(n, model)
