@@ -12,8 +12,7 @@ __all__ = [
 # IMPORTS #
 ###########
 
-
-# Major
+# Full
 
 import matplotlib.colors as mplc
 import matplotlib.image as mpli
@@ -23,7 +22,7 @@ import networkx as nx
 import numpy as np
 import numpy.linalg as npl
 
-# Minor
+# Partial
 
 from inspect import (
     trace
@@ -41,7 +40,7 @@ from subprocess import (
 # Internal
 
 from .custom_types import *
-from .exceptions import *
+from .utilities import *
 from .validation import *
 
 
@@ -49,7 +48,6 @@ from .validation import *
 # CONSTANTS #
 #############
 
-
 color_black = '#000000'
 color_gray = '#E0E0E0'
 color_white = '#FFFFFF'
@@ -60,7 +58,6 @@ colors = ['#80B1D3', '#FFED6F', '#B3DE69', '#BEBADA', '#FDB462', '#8DD3C7', '#FB
 # FUNCTIONS #
 #############
 
-
 def plot_eigenvalues(mc: tmc, dpi: int = 100) -> oplot:
 
     """
@@ -77,9 +74,8 @@ def plot_eigenvalues(mc: tmc, dpi: int = 100) -> oplot:
         mc = validate_markov_chain(mc)
         dpi = validate_dpi(dpi)
 
-    except Exception as e:
-        argument = ''.join(trace()[0][4]).split('=', 1)[0].strip()
-        raise ValidationError(str(e).replace('@arg@', argument)) from None
+    except Exception as e:  # pragma: no cover
+        raise generate_validation_error(e, trace()) from None
 
     figure, ax = pp.subplots(dpi=dpi)
 
@@ -140,14 +136,14 @@ def plot_eigenvalues(mc: tmc, dpi: int = 100) -> oplot:
 
     pp.subplots_adjust(bottom=0.2)
 
-    if pp.isinteractive():
+    if pp.isinteractive():  # pragma: no cover
         pp.show(block=False)
         return None
 
     return figure, ax
 
 
-def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges_color: bool = True, edges_value: bool = True, dpi: int = 100) -> oplot:
+def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges_color: bool = True, edges_value: bool = True, force_standard: bool = False, dpi: int = 100) -> oplot:
 
     """
     The function plots the directed graph of the Markov chain.
@@ -159,6 +155,7 @@ def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges
     :param nodes_type: a boolean indicating whether to use a different shape for every node type (by default, True).
     :param edges_color: a boolean indicating whether to display edges using a gradient based on transition probabilities, valid only for extended graphs (by default, True).
     :param edges_value: a boolean indicating whether to display the transition probability of every edge (by default, True).
+    :param force_standard: a boolean indicating whether to use a standard graph even if Graphviz and Pydot are installed (by default, False).
     :param dpi: the resolution of the plot expressed in dots per inch (by default, 100).
     :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, otherwise the handles of the plot.
     :raises ValidationError: if any input argument is not compliant.
@@ -201,28 +198,32 @@ def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges
         nodes_type = validate_boolean(nodes_type)
         edges_color = validate_boolean(edges_color)
         edges_value = validate_boolean(edges_value)
+        force_standard = validate_boolean(force_standard)
         dpi = validate_dpi(dpi)
 
-    except Exception as e:
-        argument = ''.join(trace()[0][4]).split('=', 1)[0].strip()
-        raise ValidationError(str(e).replace('@arg@', argument)) from None
+    except Exception as e:  # pragma: no cover
+        raise generate_validation_error(e, trace()) from None
 
-    extended_graph = True
-
-    # noinspection PyBroadException
-    try:
-        call(['dot', '-V'], stdout=PIPE, stderr=PIPE)
-    except Exception:
+    if force_standard:
         extended_graph = False
-        pass
+    else:
 
-    try:
-        import pydot as pyd
-    except ImportError:
-        extended_graph = False
-        pass
+        extended_graph = True
 
-    g = mc.to_directed_graph()
+        # noinspection PyBroadException
+        try:
+            call(['dot', '-V'], stdout=PIPE, stderr=PIPE)
+        except Exception:
+            extended_graph = False
+            pass
+
+        try:
+            import pydot as pyd
+        except ImportError:
+            extended_graph = False
+            pass
+
+    g = mc.to_graph()
 
     if extended_graph:
 
@@ -331,7 +332,7 @@ def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges
 
         pp.interactive(mpi)
 
-    if pp.isinteractive():
+    if pp.isinteractive():  # pragma: no cover
         pp.show(block=False)
         return None
 
@@ -364,9 +365,8 @@ def plot_redistributions(mc: tmc, distributions: tdists_flex, initial_status: os
         plot_type = validate_enumerator(plot_type, ['heatmap', 'projection'])
         dpi = validate_dpi(dpi)
 
-    except Exception as e:
-        argument = ''.join(trace()[0][4]).split('=', 1)[0].strip()
-        raise ValidationError(str(e).replace('@arg@', argument)) from None
+    except Exception as e:  # pragma: no cover
+        raise generate_validation_error(e, trace()) from None
 
     if isinstance(distributions, int):
         distributions = mc.redistribute(distributions, initial_status=initial_status, include_initial=True, output_last=False)
@@ -432,7 +432,7 @@ def plot_redistributions(mc: tmc, distributions: tdists_flex, initial_status: os
 
         pp.subplots_adjust(bottom=0.2)
 
-    if pp.isinteractive():
+    if pp.isinteractive():  # pragma: no cover
         pp.show(block=False)
         return None
 
@@ -472,9 +472,8 @@ def plot_walk(mc: tmc, walk: twalk_flex, initial_state: ostate = None, plot_type
         plot_type = validate_enumerator(plot_type, ['histogram', 'sequence', 'transitions'])
         dpi = validate_dpi(dpi)
 
-    except Exception as e:
-        argument = ''.join(trace()[0][4]).split('=', 1)[0].strip()
-        raise ValidationError(str(e).replace('@arg@', argument)) from None
+    except Exception as e:  # pragma: no cover
+        raise generate_validation_error(e, trace()) from None
 
     if isinstance(walk, int):
         walk = mc.walk(walk, initial_state=initial_state, include_initial=True, output_indices=True)
@@ -558,7 +557,7 @@ def plot_walk(mc: tmc, walk: twalk_flex, initial_state: ostate = None, plot_type
 
         ax.set_title('Walkplot (Transitions)', fontsize=15.0, fontweight='bold')
 
-    if pp.isinteractive():
+    if pp.isinteractive():  # pragma: no cover
         pp.show(block=False)
         return None
 
