@@ -5,12 +5,11 @@
 # IMPORTS #
 ###########
 
-
-# Major
+# Full
 
 import numpy as np
 
-# Minor
+# Partial
 
 from collections import (
     namedtuple
@@ -21,7 +20,8 @@ from pydtmc import (
 )
 
 from pytest import (
-    mark
+    mark,
+    skip
 )
 
 
@@ -29,7 +29,6 @@ from pytest import (
 # TEST CASES #
 ##############
 
-
 Case = namedtuple('Case', [
     'id',
     'p',
@@ -67,7 +66,7 @@ cases = [
             SubcaseFirstPassage(2, 1, 0, None, None, [1.0, 0.0], None)
         ],
         [
-            SubcaseHitting(0, [1.0, 1.0], [0.0, 1.0]),
+            SubcaseHitting(None, [1.0, 1.0], [0.0, 0.0]),
             SubcaseHitting(1, [1.0, 1.0], [1.0, 0.0])
         ],
         [
@@ -139,7 +138,7 @@ cases = [
             SubcaseFirstPassage(7, 0, 2, 1, [4.7, 1.0, 3.9], [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 32.9)
         ],
         [
-            SubcaseHitting(0, [1.0, 0.0, 0.0], [0.0, np.inf, np.inf]),
+            SubcaseHitting(None, [1.0, 1.0, 1.0], [0.0, 0.0, 0.0]),
             SubcaseHitting([1, 2], [0.0, 1.0, 1.0], [np.inf, 0.0, 0.0])
         ],
         [
@@ -247,7 +246,7 @@ cases = [
             SubcaseFirstPassage(6, 2, [0, 1], [0, 1], [5.2, 0.2, 0.0, 1.5], [0.5, 0.25, 0.0, 0.0625, 0.0, 0.015625], 4.5)
         ],
         [
-            SubcaseHitting(0, [1.0, 0.66666667, 0.33333333, 0.0], [0.0, np.inf, np.inf, np.inf]),
+            SubcaseHitting(None, [1.0, 1.0, 1.0, 1.0], [0.0, 0.0, 0.0, 0.0]),
             SubcaseHitting([0, 1], [1.0, 1.0, 0.5, 0.0], [0.0, 0.0, np.inf, np.inf])
         ],
         [
@@ -382,10 +381,9 @@ cases = [
 ]
 
 
-########
-# TEST #
-########
-
+#########
+# TESTS #
+#########
 
 @mark.parametrize(
     argnames=('p', 'absorption_probabilities'),
@@ -461,12 +459,14 @@ def test_first_passage_reward(p, steps, initial_state, first_passage_states, rew
 
     mc = MarkovChain(p)
 
-    if mc.size > 2:
+    if mc.size <= 2:
+        skip('Markov chain size is less than or equal to 2.')
+    else:
 
         actual = mc.first_passage_reward(steps, initial_state, first_passage_states, rewards)
         expected = value
 
-        assert actual == expected
+        assert np.isclose(actual, expected)
 
 
 @mark.parametrize(
@@ -516,7 +516,10 @@ def test_mean_first_passage_times_between(p, origins, targets, value):
     actual = mc.mean_first_passage_times_between(origins, targets)
     expected = value
 
-    assert actual == expected
+    if actual is not None and expected is not None:
+        assert np.isclose(actual, expected)
+    else:
+        assert actual == expected
 
 
 @mark.parametrize(
@@ -648,11 +651,14 @@ def test_mpft_and_recurrence_relation(p, mfptt_targets):
 
     mc = MarkovChain(p)
 
-    mfpt = mc.mean_first_passage_times_to(mfptt_targets)
+    if not mc.is_ergodic:
+        skip('Markov chain is not ergodic.')
+    else:
 
-    if mfpt is not None:
+        mfpt = mc.mean_first_passage_times_to(mfptt_targets)
+        mrt = mc.mean_recurrence_times()
 
         actual = mfpt
-        expected = np.dot(mc.p, mfpt) + np.ones((mc.size, mc.size), dtype=float) - np.diag(mc.mean_recurrence_times())
+        expected = np.dot(mc.p, mfpt) + np.ones((mc.size, mc.size), dtype=float) - np.diag(mrt)
 
         assert np.allclose(actual, expected)
