@@ -5,13 +5,12 @@
 # IMPORTS #
 ###########
 
-
-# Major
+# Full
 
 import numpy as np
 import numpy.linalg as npl
 
-# Minor
+# Partial
 
 from collections import (
     namedtuple
@@ -22,7 +21,8 @@ from pydtmc import (
 )
 
 from pytest import (
-    mark
+    mark,
+    skip
 )
 
 
@@ -30,7 +30,6 @@ from pytest import (
 # TEST CASES #
 ##############
 
-
 Case = namedtuple('Case', [
     'id',
     'p',
@@ -40,7 +39,10 @@ Case = namedtuple('Case', [
     'period',
     'entropy_rate', 'entropy_rate_normalized', 'topological_entropy',
     'mixing_rate', 'relaxation_rate', 'spectral_gap', 'implied_timescales',
-    'accessibility_matrix', 'adjacency_matrix', 'communication_matrix'
+    'lumping_partitions',
+    'accessibility_matrix',
+    'adjacency_matrix',
+    'communication_matrix'
 ])
 
 cases = [
@@ -53,6 +55,7 @@ cases = [
         2,
         0.0, 0.0, 0.0,
         None, None, None, None,
+        [],
         [[1, 1], [1, 1]],
         [[0, 1], [1, 0]],
         [[1, 1], [1, 1]]
@@ -66,6 +69,7 @@ cases = [
         1,
         0.36239685545027234, 0.5228281461918625, 0.6931471805599453,
         3.476059496782207, 4.0, 0.25, [np.inf, 3.4760595],
+        [],
         [[1, 1], [1, 1]],
         [[1, 1], [1, 1]],
         [[1, 1], [1, 1]]
@@ -79,6 +83,7 @@ cases = [
         1,
         0.6931471805599453, 1.0, 0.6931471805599453,
         None, None, None, [np.inf, 0.02722066],
+        [],
         [[1, 1], [1, 1]],
         [[1, 1], [1, 1]],
         [[1, 1], [1, 1]]
@@ -92,6 +97,7 @@ cases = [
         1,
         None, None, 0.0,
         None, None, None, None,
+        [],
         [[1, 0], [1, 1]],
         [[1, 0], [1, 1]],
         [[1, 0], [0, 1]]
@@ -105,6 +111,7 @@ cases = [
         1,
         None, None, 0.0,
         None, None, None, None,
+        [[[0], [1, 2]], [[0, 1], [2]]],
         [[1, 0, 0], [0, 1, 0], [0, 0, 1]],
         [[1, 0, 0], [0, 1, 0], [0, 0, 1]],
         [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
@@ -118,6 +125,7 @@ cases = [
         3,
         0.0, 0.0, 8.881784197001248e-16,
         None, None, None, None,
+        [],
         [[1, 1, 0], [0, 1, 1], [1, 0, 1]],
         [[0, 1, 0], [0, 0, 1], [1, 0, 0]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
@@ -131,6 +139,7 @@ cases = [
         1,
         0.8267342221751819, 0.8225781143836239, 1.005052538742381,
         0.8305835450825373, 1.4285714285714284, 0.7000000000000001, [np.inf, 0.83058355, 0.43429448],
+        [],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]],
         [[0, 1, 1], [1, 1, 1], [1, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
@@ -144,6 +153,7 @@ cases = [
         2,
         0.30543215102744675, 0.8812908992306926, 0.3465735902799727,
         None, None, None, None,
+        [],
         [[1, 1, 0], [1, 1, 1], [0, 1, 1]],
         [[0, 1, 0], [1, 0, 1], [0, 1, 0]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
@@ -157,6 +167,7 @@ cases = [
         1,
         0.963389503699457, 0.8769149167878063, 1.0986122886681098,
         0.6676164013906685, 1.288007155526294, 0.7763932022500208, [np.inf, 0.6676164, 0.6676164],
+        [],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
@@ -170,6 +181,7 @@ cases = [
         1,
         0.7766627625167336, 0.7069489123030946, 1.0986122886681098,
         1.293588988765789, 1.8573766183083227, 0.5383937700856752, [np.inf, 1.29358899, 1.12652175],
+        [],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
@@ -183,6 +195,7 @@ cases = [
         1,
         0.963389503699457, 0.8769149167878063, 1.0986122886681098,
         0.6676164013906685, 1.288007155526294, 0.7763932022500208, [np.inf, 0.6676164, 0.6676164],
+        [],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
@@ -196,19 +209,21 @@ cases = [
         1,
         0.5439208317104742, 0.7847118865449014, 0.6931471805599455,
         0.9876770710214384, 1.5706429060964282, 0.6366819575083007, [np.inf, 0.98767707, 0.98767707],
+        [],
         [[1, 1, 0], [0, 1, 1], [1, 0, 1]],
         [[1, 1, 0], [0, 1, 1], [1, 0, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
     ),
     Case(
         '#13',
-        [[0.5, 0.5, 0.0], [0.5, 0.25, 0.25], [0, 1/3, 2/3]],
+        [[0.5, 0.5, 0.0], [0.5, 0.25, 0.25], [0, 0.33, 0.67]],
         -0.12500000000000003, 3,
         False, True, True, True, False,
         None, None,
         1,
-        0.8037285736803539, 0.9119045380044191, 0.8813735870195428,
-        2.08276541630889, 2.622623436526918, 0.3812975915918291, [np.inf, 2.08276542, 0.62526939],
+        0.8026275636153916, 0.9106553400693125, 0.8813735870195428,
+        2.1005151596269247, 2.6400389096682244, 0.37878229610095815, [np.inf, 2.10051516, 0.6236872],
+        [],
         [[1, 1, 0], [1, 1, 1], [0, 1, 1]],
         [[1, 1, 0], [1, 1, 1], [0, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
@@ -222,6 +237,7 @@ cases = [
         1,
         0.7077237190515739, 0.8947364036861599, 0.7909857206389213,
         0.6676164013906681, 1.2880071555262935, 0.7763932022500211, [np.inf, 0.6676164, 0.6676164],
+        [],
         [[1, 1, 0], [0, 1, 1], [1, 1, 1]],
         [[0, 1, 0], [0, 1, 1], [1, 1, 1]],
         [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
@@ -235,6 +251,7 @@ cases = [
         3,
         0.23104906018664842, 0.9999999999999983, 0.2310490601866488,
         None, None, None, None,
+        [[[0], [1], [2, 3]]],
         [[1, 0, 1, 1], [1, 1, 0, 0], [0, 1, 1, 0], [0, 1, 0, 1]],
         [[0, 0, 1, 1], [1, 0, 0, 0], [0, 1, 0, 0], [0, 1, 0, 0]],
         [[1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1]]
@@ -248,6 +265,7 @@ cases = [
         1,
         0.981530111623928, 0.7080243122615159, 1.3862943611198906,
         1.043229199083855, 1.6219122507466015, 0.6165561666728137, [np.inf, 1.0432292, 0.41232175, 0.41232175],
+        [],
         [[1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1]],
         [[1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1]],
         [[1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1]]
@@ -261,6 +279,7 @@ cases = [
         3,
         0.23104906018664842, 1.0, 0.2310490601866474,
         None, None, None, None,
+        [],
         [[1, 1, 0, 0, 0], [0, 1, 1, 0, 0], [1, 0, 1, 1, 0], [0, 0, 0, 1, 1], [0, 0, 1, 0, 1]],
         [[0, 1, 0, 0, 0], [0, 0, 1, 0, 0], [1, 0, 0, 1, 0], [0, 0, 0, 0, 1], [0, 0, 1, 0, 0]],
         [[1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1]]
@@ -274,6 +293,7 @@ cases = [
         1,
         None, None, 0.6931471805599451,
         None, None, None, None,
+        [[[0, 1], [2], [3, 4, 5]], [[0], [1], [2], [3, 4, 5]], [[0, 1], [2], [3], [4], [5]]],
         [[1, 1, 0, 0, 0, 0], [1, 1, 0, 0, 0, 0], [1, 0, 1, 1, 1, 0], [0, 0, 0, 1, 1, 0], [0, 0, 0, 1, 1, 1], [0, 0, 0, 1, 0, 1]],
         [[0, 1, 0, 0, 0, 0], [1, 1, 0, 0, 0, 0], [1, 0, 1, 1, 1, 0], [0, 0, 0, 1, 1, 0], [0, 0, 0, 1, 0, 1], [0, 0, 0, 1, 0, 1]],
         [[1, 1, 0, 0, 0, 0], [1, 1, 0, 0, 0, 0], [0, 0, 1, 0, 0, 0], [0, 0, 0, 1, 1, 1], [0, 0, 0, 1, 1, 1], [0, 0, 0, 1, 1, 1]]
@@ -287,6 +307,7 @@ cases = [
         1,
         None, None, 1.4186063950217234,
         None, None, None, None,
+        [],
         [[1, 1, 1, 1, 0, 1], [1, 1, 0, 1, 0, 1], [1, 1, 1, 0, 0, 0], [1, 1, 1, 1, 0, 1], [0, 0, 0, 0, 1, 0], [1, 0, 1, 1, 0, 1]],
         [[1, 1, 1, 1, 0, 1], [1, 1, 0, 1, 0, 1], [1, 1, 0, 0, 0, 0], [1, 1, 1, 1, 0, 1], [0, 0, 0, 0, 1, 0], [1, 0, 1, 1, 0, 1]],
         [[1, 1, 1, 1, 0, 1], [1, 1, 1, 1, 0, 1], [1, 1, 1, 1, 0, 1], [1, 1, 1, 1, 0, 1], [0, 0, 0, 0, 1, 0], [1, 1, 1, 1, 0, 1]]
@@ -300,6 +321,7 @@ cases = [
         1,
         None, None, 0.4812118250596031,
         None, None, None, None,
+        [[[0, 1], [2], [3], [4], [5]]],
         [[1, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0], [1, 0, 1, 1, 0, 0], [0, 0, 1, 1, 1, 0], [0, 0, 0, 1, 1, 1], [0, 1, 0, 0, 1, 1]],
         [[1, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0], [1, 0, 0, 1, 0, 0], [0, 0, 1, 0, 1, 0], [0, 0, 0, 1, 0, 1], [0, 1, 0, 0, 1, 0]],
         [[1, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0], [0, 0, 1, 1, 1, 1], [0, 0, 1, 1, 1, 1], [0, 0, 1, 1, 1, 1], [0, 0, 1, 1, 1, 1]]
@@ -307,10 +329,9 @@ cases = [
 ]
 
 
-########
-# TEST #
-########
-
+#########
+# TESTS #
+#########
 
 @mark.parametrize(
     argnames='p',
@@ -321,7 +342,9 @@ def test_irreducibility(p):
 
     mc = MarkovChain(p)
 
-    if mc.is_irreducible:
+    if not mc.is_irreducible:
+        skip('Markov chain is not irreducible.')
+    else:
 
         actual = mc.states
         expected = mc.recurrent_states
@@ -349,7 +372,9 @@ def test_regularity(p):
 
     mc = MarkovChain(p)
 
-    if mc.is_regular:
+    if not mc.is_regular:
+        skip('Markov chain is not regular.')
+    else:
 
         actual = mc.is_irreducible
         expected = True
@@ -430,17 +455,23 @@ def test_entropy(p, entropy_rate, entropy_rate_normalized, topological_entropy):
     actual = mc.entropy_rate
     expected = entropy_rate
 
-    assert actual == expected
+    if actual is not None and expected is not None:
+        assert np.isclose(actual, expected)
+    else:
+        assert actual == expected
 
     actual = mc.entropy_rate_normalized
     expected = entropy_rate_normalized
 
-    assert actual == expected
+    if actual is not None and expected is not None:
+        assert np.isclose(actual, expected)
+    else:
+        assert actual == expected
 
     actual = mc.topological_entropy
     expected = topological_entropy
 
-    assert actual == expected
+    assert np.isclose(actual, expected)
 
 
 @mark.parametrize(
@@ -464,6 +495,24 @@ def test_fundamental_matrix(p, fundamental_matrix, kemeny_constant):
     actual = mc.kemeny_constant
     expected = kemeny_constant
 
+    if actual is not None and expected is not None:
+        assert np.isclose(actual, expected)
+    else:
+        assert actual == expected
+
+
+@mark.parametrize(
+    argnames=('p', 'lumping_partitions'),
+    argvalues=[(case.p, case.lumping_partitions) for case in cases],
+    ids=['test_lumping_partitions ' + case.id for case in cases]
+)
+def test_lumping_partitions(p, lumping_partitions):
+
+    mc = MarkovChain(p)
+
+    actual = mc.lumping_partitions
+    expected = lumping_partitions
+
     assert actual == expected
 
 
@@ -499,17 +548,26 @@ def test_times(p, mixing_rate, relaxation_rate, spectral_gap, implied_timescales
     actual = mc.mixing_rate
     expected = mixing_rate
 
-    assert actual == expected
+    if actual is not None and expected is not None:
+        assert np.isclose(actual, expected)
+    else:
+        assert actual == expected
 
     actual = mc.relaxation_rate
     expected = relaxation_rate
 
-    assert actual == expected
+    if actual is not None and expected is not None:
+        assert np.isclose(actual, expected)
+    else:
+        assert actual == expected
 
     actual = mc.spectral_gap
     expected = spectral_gap
 
-    assert actual == expected
+    if actual is not None and expected is not None:
+        assert np.isclose(actual, expected)
+    else:
+        assert actual == expected
 
     actual = mc.implied_timescales
     expected = implied_timescales
@@ -535,6 +593,17 @@ def test_binary_matrices(p, accessibility_matrix, adjacency_matrix, communicatio
 
     assert np.array_equal(actual, expected)
 
+    for i in range(mc.size):
+        for j in range(mc.size):
+
+            actual = mc.is_accessible(j, i)
+            expected = mc.accessibility_matrix[i, j] != 0
+            assert actual == expected
+
+            actual = mc.are_communicating(i, j)
+            expected = mc.accessibility_matrix[i, j] != 0 and mc.accessibility_matrix[j, i] != 0
+            assert actual == expected
+
     actual = mc.adjacency_matrix
     expected = np.asarray(adjacency_matrix)
 
